function zi_all = PolylineIntersectPolyline(pline1,pline2,varargin)
% function zi_all = PolylineIntersectPolyline(pline1,pline2,varargin)
%  Intersections of two polylines. Can be used to find 
%  intersection of two curves by approximating them as straight line segments
% 
%  A polyline is a set of connected line segments where the end point of 
%  one segment is the start of the next.
% The polyline is not necessarily closed i.e the end point is not equal the start
%  although they can be. The points of the polylines
%  are represented as complex numbers with coordinates x + iy. 
%  see the discussion at http://aprendtech.com/wordpress/?p=140
%
%  The code fully accomodates vertical lines; it does NOT use y = ax + b
%  to represent the lines. 
%  inputs:
%     pline1,pline2: 1D complex array of the start and end points of the polylines. 
%        The polylines are not necessarily equal length
%     allhits: (optional = false) If present uses loose requirements for hits. May result in dup hits
%  outputs:
%     zi_all: the intersection points of the polylines. Empty if no intersect. 
%     
%  REA: 2004-Jul-8 -> 2011-Sep-2 9:33

   % approach is to compute intersect of each segment of pline1 with all segments of pline2
nreqargs = 2;
assert(nargin>=nreqargs);
allhits = false;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'allhits';          allhits = true;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end


    % test number of lines segments in polyline
pline1 = pline1(:);    
nseg1 =  numel(pline1)-1;   
assert(nseg1>0);

pline2 = pline2(:);
nseg2 =  numel(pline2)-1;   
assert(nseg2>0);

    % find overlapping segments - from
    % http://www.mathworks.com/matlabcentral/fileexchange/11837-fast-and-robust-curve-intersections
x1 = real(pline1); y1 = imag(pline1);
x2 = real(pline2); y2 = imag(pline2);    
[idx1,idx2] = find( ... % indexes into pline1 and pline2
    repmat(min(x1(1:end-1),x1(2:end)),1,nseg2) <= ...
	repmat(max(x2(1:end-1),x2(2:end)).',nseg1,1) & ...
	repmat(max(x1(1:end-1),x1(2:end)),1,nseg2) >= ...
	repmat(min(x2(1:end-1),x2(2:end)).',nseg1,1) & ...
	repmat(min(y1(1:end-1),y1(2:end)),1,nseg2) <= ...
	repmat(max(y2(1:end-1),y2(2:end)).',nseg1,1) & ...
	repmat(max(y1(1:end-1),y1(2:end)),1,nseg2) >= ...
	repmat(min(y2(1:end-1),y2(2:end)).',nseg1,1) ...
    );
     
   % find all pairwise intersections
zi_all = [];

for k  = 1:numel(idx1)
    if allhits
       zi = PolylineIntersectSegment([pline1(idx1(k)),pline1(idx1(k)+1)], ...
           [pline2(idx2(k)),pline2(idx2(k)+1)],'allhits');
    else
       zi = PolylineIntersectSegment([pline1(idx1(k)),pline1(idx1(k)+1)], ...
           [pline2(idx2(k)),pline2(idx2(k)+1)]);
    end
       zi_all = [zi_all; zi(:)];
end
